
#include <avr/wdt.h>
#include <Wire.h>
#include "cactus_io_BME280_I2C.h"

// Create the BME280 object
BME280_I2C bme;              // I2C using default 0x77 

static volatile uint16_t an_count;
static volatile uint16_t rain_count;

float pressure_sum, humidity_sum, temp_sum;
uint32_t analog_sum;
uint16_t reading_count;

int ledPin = 13;

void setup() {
  wdt_disable();
  
  Serial.begin(2400);

  // Initialize BME280
  if (!bme.begin()) {
    Serial.println("BME280 Failure");
    while (1);
  }
  bme.setTempCal(-1);

  /* Set up INT0 and INT1 to count pulses from sensors */
  /* make sure PD2 and PD3 are in input mode */
  DDRD &= ~(1 << PD2);
  DDRD &= ~(1 << PD3);

  /* disable pull up resistor */
  PORTD &= ~(1 << PD2);
  PORTD &= ~(1 << PD3);
  
  EICRA = EICRA | (1 << ISC01);  /* set bits 1:0  to 10 (falling edge) */
  EICRA = EICRA & ~(1 << ISC00);

  EICRA = EICRA | (1 << ISC11);  /* set bits 3:2  to 10 (falling edge) */
  EICRA = EICRA & ~(1 << ISC10);

  EIMSK = EIMSK | (1 << INT0);   /* enable INT0 */
  EIMSK = EIMSK | (1 << INT1);   /* enable INT1 */

  sei(); /* enable interrupts */

  analogReference(EXTERNAL);
  
  reading_count = 0;
  pressure_sum=0.0;
  humidity_sum=0.0;
  temp_sum=0.0;

  pinMode(ledPin,OUTPUT);
  digitalWrite(ledPin, HIGH);
  delay(100);
  digitalWrite(ledPin, LOW);
  delay(100);
  digitalWrite(ledPin, HIGH);
  delay(100);
  digitalWrite(ledPin, LOW);
  
  Serial.println("NANO-WX OK");
}

void loop() {
    uint8_t command;

    if( Serial.available() > 0 )
    {
      command = Serial.read();

      if( command == 'W' )
      {
        an_count = 0;
        Serial.println("NANO-WX WIND ZERO");
      }
      if( command == 'R' )
      {
        rain_count = 0;
        Serial.println("NANO-WX RAIN ZERO");
      }
      if( command == 'D' )
      {
        if( reading_count == 0 )
        {
           bme.readSensor();  
           pressure_sum += bme.getPressure_MB();
           humidity_sum += bme.getHumidity();
           temp_sum += bme.getTemperature_F();
           analog_sum += analogRead(A0);
           reading_count++;   
        }

        Serial.print("NANO-WX DATA ");
        Serial.print(pressure_sum/reading_count); Serial.print(" ");    // Pressure in millibars
        Serial.print(humidity_sum/reading_count); Serial.print(" ");
        Serial.print(temp_sum/reading_count); Serial.print(" ");
        Serial.print(an_count); Serial.print(" ");
        Serial.print(rain_count); Serial.print(" ");
        Serial.print(analog_sum/reading_count); Serial.print(" ");
        Serial.println(reading_count);

        reading_count=0;
        pressure_sum=0.0;
        humidity_sum=0.0;
        temp_sum=0.0;
        analog_sum=0;
      }
    }

    // if number of readings in moving average gets too high, zero them out
    if( reading_count > 100 )
    {
      reading_count = 0;
      pressure_sum=0.0;
      humidity_sum=0.0;
      temp_sum=0.0;
      analog_sum=0;
    }
    
    /* read the bme280 and A0, and add readings to rolling average accumulators */
     bme.readSensor();  
     pressure_sum += bme.getPressure_MB();
     humidity_sum += bme.getHumidity();
     temp_sum += bme.getTemperature_F();
     analog_sum += analogRead(A0);
     reading_count++;   
  
    // wait a bit for command to come. meanwhile, flash LED connected to D13
    if( reading_count % 10 == 0 )
    {
      digitalWrite(ledPin, HIGH);
      delay(50);
      digitalWrite(ledPin, LOW);
    }
    else
    {
      delay(50);
    }
    delay(150);
}

// Count anemometer pulses on INT0 (PD2)
ISR(INT0_vect)
{
  an_count++;
}

// Count rain ppulses on INT1 (PD3)
ISR(INT1_vect)
{
  rain_count++;
}
